clear all 
close all
clc

%%%%%%%%%%%%%%%%%%%%% Loading Parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Temporal Sets
T = 2;    %days

%loading Excel data
data = readmatrix('node_constraints.xlsx');

%Connection Matrices
A_Aquifer = data(2:7,2);
A_Desal = data(2:7,3);
A_Pipes = data(2:7,4:11);
A_Demand = data(2:7,12:13);
A = horzcat(A_Aquifer, A_Desal, A_Pipes);
A_0 = horzcat(A,A_Demand);

%Dilution Matrix: B_0*C_0 = 0: ensures that salinities are well mixed at nodes
B_0 = data(24:27,2:13);

%Matrix Sizes
m = length(A(:,1));
n = length(A(1,:));
nA = length(A_Aquifer(1,:));
nD = length(A_Desal(1,:));
nP = length(A_Pipes(1,:));
nZ = length(A_Demand(1,:));

%Pipe Parameters 
PipeLength = data(42,4:11)';
PipeDia = data(43,4:11)';
PipeDeltaZ = data(44,4:11)';
PipeHWCoef = data(45,4:11)';
QMax = 0.21*ones(1,T);           %(MCM) based on pipe size

%Pumping Parameters
PumpingHours = 20*ones(T,1);
KHWC = 0.1*ones(T,1); 

%Demand Parameters
Demand = 0.28*ones(2,T);            %25 MCM (seasonal) / 90 days
for t = 1:T; b(:,t) = vertcat(zeros(4,1), Demand(:,t)); end 

%Desalination Parameters
DesalCapacity  = 0.5;    %roughly equal to daily demand
DesalMin       = 0;      
RemovalMin     = 99;     
RemovalMax     = 99.9;   
DesalCostAlpha = 0;      
DesalCostBeta  = 1;      

%Aquifer Extraction Parameters
AquiferDepthInitial  = 11;
AquiferDepthMax      = 100;
AquiferDepthMin      = 1;
ExtractionLevyMax    = 0;      
AquiferExtractionMax = 0.5;  %roughly equal to daily demand
 
%Hydrological Balance
Recharge    = zeros(1,T);   %no recharge - aquifer levels aren't much of a constraint at daily timestep anyways
Storativity = ones(1, T); 

%Salinity Parameters
C_MinAquifer = 0;      
C_MaxAquifer = 210;
C_MinDemand  = 0;      
C_MaxDemand  = 190;    
C_Sea        = 27000;  
C_Recharge   = 200;    
Ca_0         = 180;    

%%%%%%%%%%%%%%%% Defining Model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Declaring Decision Variables
%Flow Rates
Qp = sdpvar(repmat(nP,1,T),ones(1,T));
Qa = sdpvar(repmat(nA,1,T),ones(1,T));
Qd = sdpvar(repmat(nD,1,T),ones(1,T));
for t = 1:T; Q{t} = [Qa{t}; Qd{t}; Qp{t}]; end %combining source and pipe flows into one matrix
for t = 1:T; D{t} = [Qa{t}; Qd{t}; Qp{t}; Demand(:,1)]; end %combining all flow rates into one matrix 

%Salinity Levels
Cp = sdpvar(repmat(nP,1,T),ones(1,T));
Ca = sdpvar(repmat(nA,1,T+1),ones(1,T+1)); %one extra period in beginning to account for initial salinity
Ca{1} = Ca_0;                              %initial aquifer salinity
Cd = sdpvar(repmat(nD,1,T),ones(1,T));
Cz = sdpvar(repmat(nZ,1,T),ones(1,T));
for t = 1:T; C_0{t} = [Ca{t}; Cd{t}; Cp{t}; Cz{t}]; end %combining all flow rates into one matrix, skips initial Ca value 

%Desal removal ratio
RR = sdpvar(ones(1,T),ones(1,T));

%state variable - along with Ca
AquiferDepth = sdpvar(T+1,1);

%Cost Cefficients
CC = sdpvar(nP,1);
CE = sdpvar(nA,1);
CD = sdpvar(nD,1);

%%

%Initializing
AquiferDepth(1) = AquiferDepthInitial;
Constraints = [];
Objective = 0;
t = 0;

for t = 1:T
  

    %Conveyance cost
    Hf = (1.526*10^7) * ((Qp{t}*(10^6)./(PumpingHours(t).*PipeHWCoef)).^1.852) ...         %Hazen Williams equation
                     .* PipeLength .* (PipeDia .^ -4.87);                    
    Xp = PipeDeltaZ + Hf;                                                                  %Total head loss
    CC = (1/200) * Xp.*(Qp{t}*(10^6)./PumpingHours(t)) .* 0.736*PumpingHours(t)*KHWC(t);   %Conveyance cost per pipe

    %Aquifer Extraction cost
    CE_bar = (1 - (AquiferDepth(t+1)-AquiferDepthMin) ./ ...
                  (AquiferDepthMax-AquiferDepthMin)) .* ExtractionLevyMax;   %Extraction levy for given aquifer
    CE = CE_bar * Qa{t}*(10^6);                                                 

    %Desalination Cost
    CD = (DesalCostAlpha + 1 / ((100-RR{t}).^DesalCostBeta)) .* Qd{t}*10^6;
    
    %Constraints 
    Constraints = [Constraints,...
                   A*Q{t} == b(:,t),...              %conservation of mass for water at nodes                  
                   A_0*diag(D{t})*C_0{t} == 0,...    %conservation of mass for salinity at nodes
                   B_0*C_0{t} == 0,...               %dilution condition
                   Storativity(t)*( AquiferDepth(t+1)-AquiferDepth(t)) + Qa{t} == Recharge(t),... %hydrological balance
                   Storativity(t)*(Ca{t+1}*AquiferDepth(t+1) - Storativity(t)*Ca{t}*AquiferDepth(t)) + Ca{t}*Qa{t} == C_Recharge*Recharge(t),... %salinity balance
                   Cd{t} + (C_Sea/100)*RR{t} == C_Sea,...                  %salinity of desal
                   (0 <= Qp{t}) & (Qp{t} <= QMax(t)),...                   %pipe flow constraints                      
                   (0 <= Qa{t}) & (Qa{t} <= AquiferExtractionMax),...      %aquifer extraction constraints
                   (DesalMin <= Qd{t}) & (Qd{t} <= DesalCapacity),...      %desal flow constraints
                   (RemovalMin <= RR{t}) & (RR{t} <= RemovalMax),...
                   (C_MinAquifer <= Ca{t+1}) & (Ca{t+1} <= C_MaxAquifer),...
                   (C_MinDemand <= Cz{t}) & (Cz{t} <= C_MaxDemand),...
                   C_0{t} >= 0,...
                   (AquiferDepthMin <= AquiferDepth(t+1)) & (AquiferDepth(t+1) <= AquiferDepthMax)];  %aquifer level constraints
          

    %Objective Function          
    Objective = Objective + sum(CC) + sum(CD) + sum(CE);
%         Objective = Objective + (sum(CC) + sum(CD) + sum(CE))/(1+DiscountRate)^y 
        
    
end

%% Solving Model
tic
ops = sdpsettings('solver','fmincon','debug',1);
ops.scip.solverOpts = scipset('scipopts',{'limits/gap',0.1});
% ops.scip.solverOpts = scipset('scipopts',{'parallel/mode',1});  
% ops.scip.solverOpts = scipset('scipopts',{'parallel/minnthreads',6});  %use all 6 cores
ops.fmincon.TolFun = 1e-6;
optimize(Constraints,Objective,ops);
toc

%% Printing Final Results
cost_result = value(Objective);
flow_results =     [];
salinity_results = [];
aquifer_results =  [];
RR_results =       [];

for t = 1:T 
    flow_results =     [flow_results Q{t}];
    salinity_results = [salinity_results C_0{t}];
    aquifer_results =  [aquifer_results AquiferDepth(t+1)];
    RR_results =       [RR_results RR{t}];
end
     
disp(' ')
disp('Salinities for each season/year:')
disp([num2str(value(salinity_results)')])
disp(' ')
disp('Flow rates for each season/year:')
disp([num2str(value(flow_results)')])
disp(' ')
disp(['The optimal cost is $', num2str(cost_result)])
disp(' ')

   

